package com.ruoyi.project.check.service;

import java.util.List;
import com.ruoyi.project.check.domain.TCheckCalibration;

/**
 * 检测仪器校准Service接口
 * 
 * @author ruoyi
 * @date 2022-11-23
 */
public interface ITCheckCalibrationService 
{
    /**
     * 查询检测仪器校准
     * 
     * @param id 检测仪器校准主键
     * @return 检测仪器校准
     */
    public TCheckCalibration selectTCheckCalibrationById(Long id);

    /**
     * 查询检测仪器校准列表
     * 
     * @param tCheckCalibration 检测仪器校准
     * @return 检测仪器校准集合
     */
    public List<TCheckCalibration> selectTCheckCalibrationList(TCheckCalibration tCheckCalibration);

    /**
     * 新增检测仪器校准
     * 
     * @param tCheckCalibration 检测仪器校准
     * @return 结果
     */
    public int insertTCheckCalibration(TCheckCalibration tCheckCalibration);

    /**
     * 修改检测仪器校准
     * 
     * @param tCheckCalibration 检测仪器校准
     * @return 结果
     */
    public int updateTCheckCalibration(TCheckCalibration tCheckCalibration);

    /**
     * 批量删除检测仪器校准
     * 
     * @param ids 需要删除的检测仪器校准主键集合
     * @return 结果
     */
    public int deleteTCheckCalibrationByIds(Long[] ids);

    /**
     * 删除检测仪器校准信息
     * 
     * @param id 检测仪器校准主键
     * @return 结果
     */
    public int deleteTCheckCalibrationById(Long id);
}
